#include <stdlib.h> 
#include <getopt.h>
#include <iostream>
#include <fstream>
#include <list>
#include <limits>
using namespace std;

// Задача о ранце: https://ru.wikipedia.org/wiki/%D0%97%D0%B0%D0%B4%D0%B0%D1%87%D0%B0_%D0%BE_%D1%80%D0%B0%D0%BD%D1%86%D0%B5

int debug_level = 0;                           // уровень отладосного вывода

struct entity {
   bool bGood;
   double cost, weight;                        // стоимость и вес
   entity( double cost = 0, double weight = 0 ) { this->cost = cost; this->weight = weight; }
   inline entity& operator +=( entity& add ) {
      this->cost += add.cost;
      this->weight += add.weight;
      return *this;
   }
   friend inline ostream& operator << ( ostream& stream, entity obj ) {
      return stream << "{" <<  obj.cost << ":" << obj.weight << "}"; 
   };
   friend istream& operator >> ( istream& stream, entity& obj );
};

inline istream& operator >> ( istream& stream, entity& obj ) {
   double c, w;
   string s;
   obj.bGood = false;                          // флаг неправильного ввода 
   if( ( stream >> c ).eof() ) return stream;  // ввод cost
   if( stream.rdstate() & ios::failbit ) {
      cerr << "ошибка ввода!" << endl;
      stream.clear();
      getline( stream, s );
      return stream;
   }
   if( c <= 0 ) {
      cerr << "ошибка ввода: " << c << endl;
      return stream;
   }   
   if( ( stream >> w ).eof() ) return stream;  // ввод weight
   if( stream.rdstate() & ios::failbit ) {
      cerr << "ошибка ввода!" << endl;
      stream.clear();
      getline( stream, s );
      return stream;
   }
   if( w <= 0 ) {
      cerr << "ошибка ввода: " << w << endl;
      return stream;
   }   
   getline( stream, s );
   if( !s.empty() ) {                          // если введено больше 2-значений
      basic_string<char>::iterator i;
      for( i = s.begin(); i != s.end() && *i == ' '; i++ );
      if( i != s.end() ) {                     // если там непробельные символы
         cerr << "ошибка ввода: " << s << endl;
         return stream;
      }
   }
   obj = entity( c, w );
   obj.bGood = true;
   return stream;
};

//---------------------------------- end struct entity ------------------------------

class set : public list<entity> {
private:
   set *result;
   set packlvl( int level, double volume, double& cost, set& globl );
public:
   static ulong counter;
   friend set packlvl( int level, double volume, set& list );
   set( void ) : list<entity>() {};            // пустой список предметов
   set( char *file );                          // ввод писка предметов
   entity summa( void ) {
      entity res;
      for( list<entity>::iterator i = begin(); i != end(); i++ ) 
         res += *i;
      return entity( res );
   }
   set pack( double volume ) { 
      counter = 1;
      set ret;
      double cost = 0;
      packlvl( 0, volume, cost, ret );
      return ret;
   }
   friend ostream& operator <<( ostream& stream, set* obj );
   friend inline ostream& operator <<( ostream& stream, set& obj ) {
      return stream << &obj;
   }
};

ulong set::counter;                            // счётчик перебранных вариантов

set::set( char *file ) {                       // ввод полного набора предметов
   entity data;
   if( file != NULL ) {
      ifstream fin;
      fin.open( file );
      if( !fin ) {
         cerr << "ошибка открытия " << file << endl;
         exit( 1 );
      }
      while( true ) {
         fin >> data;
         if( fin.eof() ) break;
         if( data.bGood ) push_back( data );
      }
      fin.close();
   }
   else {
      cout << "вводите пары чисел (стоимость, вес) через пробел (^D для завершения):" << endl;
      while( true ) {
         cin >> data;
         if( cin.eof() ) break;
         if( data.bGood ) push_back( data );
      }
   }
}

ostream& operator << ( ostream& stream, set* obj ) {
   if( obj->empty() )
      return stream << "{}";
   for( list<entity>::iterator i = obj->begin(); i != obj->end(); i++ ) 
      stream << *i << " ";
   return stream;
}

set set::packlvl( int level, double volume, double& cost, set& globl ) {
   set ret;
   level++;
   if( debug_level > 1 ) {
      for( int i = 0; i < level; i++ ) cout << "----";
      cout << "> " << this << endl;
   }
   for( list<entity>::iterator i = begin(); i != end(); i++ ) {
      set child;
      for( list<entity>::iterator j = begin(); j != end(); j++ ) 
         if( j != i ) child.push_back( *j );
      if( child.empty() ) ret = child;
      else {
         ret = child.packlvl( level, volume, cost, globl );
         counter++;
      }
      ret.push_front( *i );
      entity sum = summa();
      bool next = sum.weight <= volume && sum.cost > cost;
      if( debug_level > 2 ) {
         cout << '<';
         for( int i = 0; i < level; i++ ) cout << "----";
         cout << " " << ret << "| cost=" << cost << " "
              << sum << ( next ? "+"  : "-" ) << endl;
      }
      if( sum.weight <= volume && sum.cost > cost ) {
         cost = sum.cost;
         globl = ret;
      } 
   }
   level--;
   return ret;
}

//------------------------------------ end class set --------------------------------

int main( int argc, char **argv, char **envp ) {
   char c;
   while( -1 != ( c = getopt( argc, argv, "hv" ) ) )
      switch( c ) {
         case 'v':
            debug_level++;
            break;
         case 'h':
         default :
            cerr << "запуск: " << argv[ 0 ] 
                 << " [-v...][-h] <вместительность> [<файл предметов>]" << endl;
            if( c != 'h' ) return( EXIT_FAILURE );
      }
   if( argc - optind < 1 || argc - optind > 2 ) // 1 или 2 параметра
      cerr << "запуск: " << argv[ 0 ] 
           << "[-v...][-h] <вместительность> [<файл предметов>]" << endl,
      exit( EXIT_FAILURE );
   double volume = atof( argv[ optind ] );
   set arr( argc - optind > 1 ? argv[ optind + 1 ] : NULL );
   cout << "набор [" << arr.size() << "]: " << arr
        << "при вместительности " << volume << endl;
   if( debug_level > 0 ) cout << "суммарно весь набор = " << arr.summa() << endl; 
   set res = arr.pack( volume );
   cout << "решение [" << res.size() << "]: " << res 
        << "- результат: " << res.summa() << endl;  
   if( debug_level > 0 ) cout << "число циклов перебора = " << set::counter << endl;
   return 0;
}

